@@ -539,7 +539,7 @@ ephy_about_handler_handle_incognito (EphyAboutHandler *handler,
                           "    <div style=\"background: transparent url(ephy-resource:///org/gnome/epiphany/incognito.png) no-repeat 10px center;\">\n" \
                           "      <h1>%s</h1>\n"
                           "      <p>%s</p>\n"
-                          "      <p><strong>%s</strong></p>\n"
+                          "      <p><strong>%s</strong> %s</p>\n"
                           "    </div>\n"
                           "  </div>\n"
                           "</body>\n"
@@ -552,9 +552,12 @@ ephy_about_handler_handle_incognito (EphyAboutHandler *handler,
                             "mode will not show up in your browsing history and all stored "
                             "information will be cleared when you close the window. Files you "
                             "download will be kept."),
-                          _("Incognito mode will not hide your activity from your employer, "
-                            "your Internet Service Provider, your government, or the websites "
-                            "that you visit."));
+                          _("Incognito mode hides your activity only from people using this "
+                            "computer."),
+                         _("It will not hide your activity from your employer if you are at "
+                            "work. Your internet service provider, your government, other "
+                            "governments, the websites that you visit, and advertisers on "
+                            "these websites may still be tracking you."));
 
   ephy_about_handler_finish_request (request, data, -1);
 
