@@ -31,6 +31,7 @@
 #include "ephy-dnd.h"
 #include "ephy-location-entry.h"
 #include "ephy-shell.h"
+#include "ephy-title-box.h"
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
@@ -49,6 +50,7 @@ struct _EphyLocationControllerPrivate
 {
 	EphyWindow *window;
 	EphyLocationEntry *location_entry;
+	EphyTitleBox *title_box;
 	GList *actions;
 	char *address;
 	EphyNode *smart_bmks;
@@ -76,7 +78,8 @@ enum
 	PROP_ICON,
 	PROP_SHOW_ICON,
 	PROP_WINDOW,
-	PROP_LOCATION_ENTRY
+	PROP_LOCATION_ENTRY,
+	PROP_TITLE_BOX
 };
 
 enum
@@ -257,17 +260,13 @@ user_changed_cb (GtkWidget *widget, EphyLocationController *controller)
 
 	address = ephy_location_entry_get_location (EPHY_LOCATION_ENTRY (widget));
 
-	LOG ("user_changed_cb, new address %s", address);
-
-	g_signal_handlers_block_by_func (controller, G_CALLBACK (sync_address), widget);
-	ephy_location_controller_set_address (controller, address);
+	LOG ("user_changed_cb, address %s", address);
 
 	completion = gtk_entry_get_completion (GTK_ENTRY (widget));
 	model = gtk_entry_completion_get_model (completion);
 
 	ephy_completion_model_update_for_string (EPHY_COMPLETION_MODEL (model), address,
 						 update_done_cb, completion);
-	g_signal_handlers_unblock_by_func (controller, G_CALLBACK (sync_address), widget);
 }
 
 static void
@@ -289,9 +288,21 @@ sync_address (EphyLocationController *controller,
 
 	g_signal_handlers_block_by_func (widget, G_CALLBACK (user_changed_cb), controller);
 	ephy_location_entry_set_location (lentry, priv->address);
+	ephy_title_box_set_address (priv->title_box, priv->address);
 	g_signal_handlers_unblock_by_func (widget, G_CALLBACK (user_changed_cb), controller);
 }
 
+static void
+title_box_mode_changed_cb (EphyTitleBox *title_box,
+			   GParamSpec *psec,
+			   gpointer user_data)
+{
+	EphyLocationController *controller = EPHY_LOCATION_CONTROLLER (user_data);
+	EphyLocationControllerPrivate *priv = controller->priv;
+
+	sync_address (controller, NULL, GTK_WIDGET (priv->location_entry));
+}
+
 static char *
 get_location_cb (EphyLocationEntry *entry,
 		EphyLocationController *controller)
@@ -446,6 +457,9 @@ ephy_location_controller_constructed (GObject *object)
 
 	add_completion_actions (controller, priv->location_entry);
 
+	g_signal_connect_object (priv->title_box, "notify::mode",
+				 G_CALLBACK (title_box_mode_changed_cb), controller, 0);
+
 	sync_address (controller, NULL, widget);
 	g_signal_connect_object (controller, "notify::address",
 				 G_CALLBACK (sync_address), widget, 0);
@@ -514,6 +528,9 @@ ephy_location_controller_set_property (GObject *object,
 		case PROP_LOCATION_ENTRY:
 			priv->location_entry = EPHY_LOCATION_ENTRY (g_value_get_object (value));
 			break;
+		case PROP_TITLE_BOX:
+			priv->title_box = EPHY_TITLE_BOX (g_value_get_object (value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 	}
@@ -565,6 +582,8 @@ ephy_location_controller_dispose (GObject *object)
 					      0, 0, NULL, NULL, priv->location_entry);
 	g_signal_handlers_disconnect_matched (priv->location_entry, G_SIGNAL_MATCH_DATA,
 					      0, 0, NULL, NULL, controller);
+	g_signal_handlers_disconnect_matched (priv->title_box, G_SIGNAL_MATCH_DATA,
+					      0, 0, NULL, NULL, controller);
 	g_signal_handlers_disconnect_matched (notebook, G_SIGNAL_MATCH_DATA,
 					      0, 0, NULL, NULL, controller);
 	priv->location_entry = NULL;
@@ -680,6 +699,20 @@ ephy_location_controller_class_init (EphyLocationControllerClass *class)
 							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB |
 							      G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	* EphyLocationController:title-box:
+	*
+	* The #EphyLocationController sets the address of this title box.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_TITLE_BOX,
+					 g_param_spec_object ("title-box",
+							      "Title box",
+							      "The title box whose address will be managed",
+							      G_TYPE_OBJECT,
+							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB |
+							      G_PARAM_CONSTRUCT_ONLY));
+
 	g_type_class_add_private (object_class, sizeof (EphyLocationControllerPrivate));
 }
 
